/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _mainframe_
#define _mainframe_

#include "wx/wx.h"
#include "wx/scrolbar.h"
#include "wx/image.h"
#include "wx/spinctrl.h"

#include "wxAdditions/bsizer.h"

#include "Config.h"

#include "ptr_vector.h"

namespace AriaMaestosa {

enum {
	PLAY_ALWAYS = 0,
	PLAY_ON_CHANGE = 1,
	PLAY_NEVER = 2
};

class GLPane; // forward
class MusicPlayer;
class CustomNoteSelectDialog;
class Sequence;
class Preferences;
class AboutDialog;

class MainFrame : public wxFrame
{
    
	DECLARE_LEAK_CHECK();
	
	AboutDialog* aboutDialog;
	
    wxBorderSizer* verticalSizer;
    wxPanel* topPane;
    wxBoxSizer* boxSizer;
    
    wxScrollBar* horizontalScrollbar;
    wxScrollBar* verticalScrollbar;
    
    wxTextCtrl* measureTypeBottom;
    wxTextCtrl* measureTypeTop;
    wxTextCtrl* firstMeasure;
    wxSpinCtrl* songLength;
    wxSpinCtrl* displayZoom;
    
    CustomNoteSelectDialog* customNoteSelectDialog;
    Preferences* prefs;
	
    wxMenu* fileMenu;
    wxMenu* editMenu;
    wxMenu* trackMenu;
	wxMenu* helpMenu;
    
	wxMenuItem* followPlaybackMenuItem;
	
	wxMenuItem* playDuringEdits_always;
	wxMenuItem* playDuringEdits_onchange;
	wxMenuItem* playDuringEdits_never;
	
	int currentSequence;
    ptr_vector<Sequence> sequences; // contains all open sequences
	
    
    wxBitmapButton* play;
    wxBitmapButton* stop;
	
public:
		
	// READ AND WRITE
	bool changingValues; // set this to true when modifying the controls in the top bar, this allows to ignore all events thrown by their modification.
	wxTextCtrl* tempoCtrl;
	
    // ------- read-only -------
	int play_during_edit; // what is the user's preference for note preview during edits
	bool playback_mode;
    GLPane* glPane;
    // ----------------------
        
    MainFrame();
    ~MainFrame();
    
    // top bar controls updated
    void tempoChanged(wxCommandEvent& evt);
    void songLengthChanged(wxSpinEvent& evt);
    void zoomChanged(wxSpinEvent& evt);
    void songLengthTextChanged(wxCommandEvent& evt);
    void zoomTextChanged(wxCommandEvent& evt);
    void measureNumChanged(wxCommandEvent& evt);
    void measureDenomChanged(wxCommandEvent& evt);
    void firstMeasureChanged(wxCommandEvent& evt);
	void changeMeasureAmount(int i, bool throwEvent=true);
	
	// menus
    void menuEvent_new(wxCommandEvent& evt);
    void menuEvent_close(wxCommandEvent& evt);
	void menuEvent_exportNotation(wxCommandEvent& evt);
    void menuEvent_open(wxCommandEvent& evt);
    void menuEvent_copy(wxCommandEvent& evt);
    void menuEvent_undo(wxCommandEvent& evt);
    void menuEvent_paste(wxCommandEvent& evt);
    void menuEvent_pasteAtMouse(wxCommandEvent& evt);
    void menuEvent_save(wxCommandEvent& evt);
    void menuEvent_saveas(wxCommandEvent& evt);
    void menuEvent_selectNone(wxCommandEvent& evt);
    void menuEvent_selectAll(wxCommandEvent& evt);
    void menuEvent_addTrack(wxCommandEvent& evt);
    void menuEvent_deleteTrack(wxCommandEvent& evt);
    void menuEvent_importmidi(wxCommandEvent& evt);
    void menuEvent_exportmidi(wxCommandEvent& evt);
    void menuEvent_exportSampledAudio(wxCommandEvent& evt);
    void menuEvent_customNoteSelect(wxCommandEvent& evt);
    void menuEvent_snapToGrid(wxCommandEvent& evt);
    void menuEvent_scale(wxCommandEvent& evt);
    void menuEvent_copyright(wxCommandEvent& evt);
	void menuEvent_preferences(wxCommandEvent& evt);
	void menuEvent_followPlayback(wxCommandEvent& evt);
	void menuEvent_removeOverlapping(wxCommandEvent& evt);
	void menuEvent_playAlways(wxCommandEvent& evt);
	void menuEvent_playOnChange(wxCommandEvent& evt);
	void menuEvent_playNever(wxCommandEvent& evt);
    void menuEvent_quit(wxCommandEvent& evt);
	void menuEvent_about(wxCommandEvent& evt);
    void menuEvent_manual(wxCommandEvent& evt);	
	
    // playback
    void songHasFinishedPlaying();
    void toolsEnterPlaybackMode();
    void toolsExitPlaybackMode();
	void playClicked(wxCommandEvent& evt);
    void stopClicked(wxCommandEvent& evt);
	
	// i/o
	void updateTopBarForSequence(Sequence* seq);
	void loadAriaFile(wxString path);
	void loadMidiFile(wxString path);
	
	// scrollbars
    void updateVerticalScrollbar();
    void setHorizontalScrollbar(int thumbPos);
	void horizontalScrolling(wxScrollEvent& evt);
    void verticalScrolling(wxScrollEvent& evt);
    void horizontalScrolling_arrows(wxScrollEvent& evt);
    void verticalScrolling_arrows(wxScrollEvent& evt);
	
	// sequences
	void addSequence();
    void selectSequence(int id);
    int getSequenceAmount();
    bool closeSequence(int id=-1); // -1 means current sequence
    Sequence* getCurrentSequence();
	int getCurrentSequenceID();
	void setCurrentSequence(int n);
	Sequence* getSequence(int n);
    
    DECLARE_EVENT_TABLE();
};

}

#endif
